@@ -24,12 +24,9 @@
  *
  * 4. This notice may not be removed or altered.
  */
-#include "file.h"
-#ifdef __CYGWIN__
-#include <errno.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/param.h>	/* for MAXPATHLEN */
@@ -49,20 +46,24 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* for read() */
 #endif
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 #include <netinet/in.h>		/* for byte swapping */
 
+#include "file.h"
 #include "patchlevel.h"
 
 #ifndef	lint
-FILE_RCSID("@(#)$Id: file.c,v 1.47 1999/10/31 22:23:03 christos Exp $")
+FILE_RCSID("@(#)$Id: file.c,v 1.56 2001/03/11 20:29:16 christos Exp $")
 #endif	/* lint */
 
 
 #ifdef S_IFLNK
-# define USAGE  "Usage: %s [-bcnvzL] [-f namefile] [-m magicfiles] file...\n"
+# define USAGE  "Usage: %s [-bciknvzL] [-f namefile] [-m magicfiles] file...\n"
 #else
-# define USAGE  "Usage: %s [-bcnvz] [-f namefile] [-m magicfiles] file...\n"
+# define USAGE  "Usage: %s [-bciknvz] [-f namefile] [-m magicfiles] file...\n"
 #endif
 
 #ifndef MAGIC
@@ -79,19 +80,24 @@ int 			/* Global command-line options 		*/
 	bflag = 0,	/* brief output format	 		*/
 	zflag = 0,	/* follow (uncompress) compressed files */
 	sflag = 0,	/* read block special files		*/
-	nobuffer = 0;   /* Do not buffer stdout */
+	iflag = 0,
+	nobuffer = 0,   /* Do not buffer stdout */
+	kflag = 0;	/* Keep going after the first match	*/
+
 int			/* Misc globals				*/
 	nmagic = 0;	/* number of valid magic[]s 		*/
 
 struct  magic *magic;	/* array of magic entries		*/
 
 const char *magicfile;	/* where magic be found 		*/
+const char *default_magicfile = MAGIC;
 
 char *progname;		/* used throughout 			*/
 int lineno;		/* line number in the magic file	*/
 
 
 static void	unwrap		__P((char *fn));
+static void	usage		__P((void));
 #if 0
 static int	byteconv4	__P((int, int, int));
 static short	byteconv2	__P((int, int, int));
@@ -108,7 +114,12 @@ main(argc, argv)
 	char *argv[];
 {
 	int c;
-	int check = 0, didsomefiles = 0, errflg = 0, ret = 0, app = 0;
+	int action = 0, didsomefiles = 0, errflg = 0, ret = 0, app = 0;
+	char *mime;
+
+#ifdef LC_CTYPE
+	setlocale(LC_CTYPE, ""); /* makes islower etc work for other langs */
+#endif
 
 	if ((progname = strrchr(argv[0], '/')) != NULL)
 		progname++;
@@ -116,52 +127,66 @@ main(argc, argv)
 		progname = argv[0];
 
 	if (!(magicfile = getenv("MAGIC")))
-		magicfile = MAGIC;
+		magicfile = default_magicfile;
 
-	while ((c = getopt(argc, argv, "bcdnf:m:svzL")) != EOF)
+	while ((c = getopt(argc, argv, "bcdf:ikm:nsvzCL")) != EOF)
 		switch (c) {
-		case 'v':
-			(void) fprintf(stdout, "%s-%d.%d\n", progname,
-				       FILE_VERSION_MAJOR, patchlevel);
-			(void) fprintf(stdout, "magic file from %s\n",
-				       magicfile);
-			return 1;
 		case 'b':
 			++bflag;
 			break;
 		case 'c':
-			++check;
+			action = CHECK;
 			break;
-		case 'n':
-			++nobuffer;
+		case 'C':
+			action = COMPILE;
 			break;
 		case 'd':
 			++debug;
 			break;
 		case 'f':
 			if (!app) {
-				ret = apprentice(magicfile, check);
-				if (check)
+				ret = apprentice(magicfile, action);
+				if (action)
 					exit(ret);
 				app = 1;
 			}
 			unwrap(optarg);
 			++didsomefiles;
 			break;
+		case 'i':
+			iflag++;
+			if ((mime = malloc(strlen(magicfile) + 5)) != NULL) {
+				(void)strcpy(mime, magicfile);
+				(void)strcat(mime, ".mime");
+				magicfile = mime;
+			}
+			break;
+		case 'k':
+			kflag = 1;
+			break;
+		case 'm':
+			magicfile = optarg;
+			break;
+		case 'n':
+			++nobuffer;
+			break;
+		case 's':
+			sflag++;
+			break;
+		case 'v':
+			(void) fprintf(stdout, "%s-%d.%d\n", progname,
+				       FILE_VERSION_MAJOR, patchlevel);
+			(void) fprintf(stdout, "magic file from %s\n",
+				       magicfile);
+			return 1;
+		case 'z':
+			zflag++;
+			break;
 #ifdef S_IFLNK
 		case 'L':
 			++lflag;
 			break;
 #endif
-		case 'm':
-			magicfile = optarg;
-			break;
-		case 'z':
-			zflag++;
-			break;
-		case 's':
-			sflag++;
-			break;
 		case '?':
 		default:
 			errflg++;
@@ -169,21 +194,19 @@ main(argc, argv)
 		}
 
 	if (errflg) {
-		(void) fprintf(stderr, USAGE, progname);
-		exit(2);
+		usage();
 	}
 
 	if (!app) {
-		ret = apprentice(magicfile, check);
-		if (check)
+		ret = apprentice(magicfile, action);
+		if (action)
 			exit(ret);
 		app = 1;
 	}
 
 	if (optind == argc) {
 		if (!didsomefiles) {
-			(void)fprintf(stderr, USAGE, progname);
-			exit(2);
+			usage();
 		}
 	}
 	else {
@@ -206,7 +229,7 @@ main(argc, argv)
  */
 static void
 unwrap(fn)
-char *fn;
+	char *fn;
 {
 	char buf[MAXPATHLEN];
 	FILE *f;
@@ -251,29 +274,28 @@ char *fn;
  */
 static int
 byteconv4(from, same, big_endian)
-    int from;
-    int same;
-    int big_endian;
+	int from;
+	int same;
+	int big_endian;
 {
-  if (same)
-    return from;
-  else if (big_endian)		/* lsb -> msb conversion on msb */
-  {
-    union {
-      int i;
-      char c[4];
-    } retval, tmpval;
+	if (same)
+		return from;
+	else if (big_endian) {		/* lsb -> msb conversion on msb */
+		union {
+			int i;
+			char c[4];
+		} retval, tmpval;
 
-    tmpval.i = from;
-    retval.c[0] = tmpval.c[3];
-    retval.c[1] = tmpval.c[2];
-    retval.c[2] = tmpval.c[1];
-    retval.c[3] = tmpval.c[0];
+		tmpval.i = from;
+		retval.c[0] = tmpval.c[3];
+		retval.c[1] = tmpval.c[2];
+		retval.c[2] = tmpval.c[1];
+		retval.c[3] = tmpval.c[0];
 
-    return retval.i;
-  }
-  else
-    return ntohl(from);		/* msb -> lsb conversion on lsb */
+		return retval.i;
+	}
+	else
+		return ntohl(from);	/* msb -> lsb conversion on lsb */
 }
 
 /*
@@ -286,23 +308,22 @@ byteconv2(from, same, big_endian)
 	int same;
 	int big_endian;
 {
-  if (same)
-    return from;
-  else if (big_endian)		/* lsb -> msb conversion on msb */
-  {
-    union {
-      short s;
-      char c[2];
-    } retval, tmpval;
+	if (same)
+		return from;
+	else if (big_endian) {		/* lsb -> msb conversion on msb */
+		union {
+			short s;
+			char c[2];
+		} retval, tmpval;
 
-    tmpval.s = (short) from;
-    retval.c[0] = tmpval.c[1];
-    retval.c[1] = tmpval.c[0];
+		tmpval.s = (short) from;
+		retval.c[0] = tmpval.c[1];
+		retval.c[1] = tmpval.c[0];
 
-    return retval.s;
-  }
-  else
-    return ntohs(from);		/* msb -> lsb conversion on lsb */
+		return retval.s;
+	}
+	else
+		return ntohs(from);	/* msb -> lsb conversion on lsb */
 }
 #endif
 
@@ -311,8 +332,8 @@ byteconv2(from, same, big_endian)
  */
 void
 process(inname, wid)
-const char	*inname;
-int wid;
+	const char	*inname;
+	int wid;
 {
 	int	fd = 0;
 	static  const char stdname[] = "standard input";
@@ -335,22 +356,22 @@ int wid;
 			   (int) (wid - strlen(inname)), "");
 
 	if (inname != stdname) {
-	    /*
-	     * first try judging the file based on its filesystem status
-	     */
-	    if (fsmagic(inname, &sb) != 0) {
-		    putchar('\n');
-		    return;
-	    }
+		/*
+		 * first try judging the file based on its filesystem status
+		 */
+		if (fsmagic(inname, &sb) != 0) {
+			putchar('\n');
+			return;
+		}
 
-	    if ((fd = open(inname, O_RDONLY)) < 0) {
-		    /* We can't open it, but we were able to stat it. */
-		    if (sb.st_mode & 0002) ckfputs("writeable, ", stdout);
-		    if (sb.st_mode & 0111) ckfputs("executable, ", stdout);
-		    ckfprintf(stdout, "can't read `%s' (%s).\n",
-			inname, strerror(errno));
-		    return;
-	    }
+		if ((fd = open(inname, O_RDONLY)) < 0) {
+			/* We can't open it, but we were able to stat it. */
+			if (sb.st_mode & 0002) ckfputs("writeable, ", stdout);
+			if (sb.st_mode & 0111) ckfputs("executable, ", stdout);
+			ckfprintf(stdout, "can't read `%s' (%s).\n",
+			    inname, strerror(errno));
+			return;
+		}
 	}
 
 
@@ -363,15 +384,24 @@ int wid;
 	}
 
 	if (nbytes == 0)
-		ckfputs("empty", stdout);
+		ckfputs(iflag ? "application/x-empty" : "empty", stdout);
 	else {
 		buf[nbytes++] = '\0';	/* null-terminate it */
 		match = tryit(buf, nbytes, zflag);
 	}
 
 #ifdef BUILTIN_ELF
-	if (match == 's' && nbytes > 5)
+	if (match == 's' && nbytes > 5) {
+		/*
+		 * We matched something in the file, so this *might*
+		 * be an ELF file, and the file is at least 5 bytes long,
+		 * so if it's an ELF file it has at least one byte
+		 * past the ELF magic number - try extracting information
+		 * from the ELF headers that can't easily be extracted
+		 * with rules in the magic file.
+		 */
 		tryelf(fd, buf, nbytes);
+	}
 #endif
 
 	if (inname != stdname) {
@@ -404,8 +434,8 @@ int wid;
 
 int
 tryit(buf, nb, zflag)
-unsigned char *buf;
-int nb, zflag;
+	unsigned char *buf;
+	int nb, zflag;
 {
 	/* try compression stuff */
 	if (zflag && zmagic(buf, nb))
@@ -419,11 +449,17 @@ int nb, zflag;
 	if (ascmagic(buf, nb))
 		return 'a';
 
-	/* see if it's international language text */
-	if (internatmagic(buf, nb))
-		return 'i';
-
 	/* abandon hope, all ye who remain here */
 	ckfputs("data", stdout);
 		return '\0';
 }
+
+static void
+usage()
+{
+	(void)fprintf(stderr, USAGE, progname);
+#ifdef QUICK
+	(void)fprintf(stderr, "Usage: %s -C [-m magic]\n", progname);
+#endif
+	exit(1);
+}
