@@ -41,14 +41,14 @@
 #include <time.h>
 
 #ifndef lint
-FILE_RCSID("@(#)$Id: print.c,v 1.29 1999/10/31 22:23:04 christos Exp $")
+FILE_RCSID("@(#)$Id: print.c,v 1.32 2001/03/11 20:29:16 christos Exp $")
 #endif  /* lint */
 
 #define SZOF(a)	(sizeof(a) / sizeof(a[0]))
 
 void
 mdump(m)
-struct magic *m;
+	struct magic *m;
 {
 	static const char *typ[] = { "invalid", "byte", "short", "invalid",
 				     "long", "string", "date", "beshort",
@@ -61,50 +61,62 @@ struct magic *m;
 	if (m->flag & INDIR)
 		(void) fprintf(stderr, "(%s,%d),",
 			       /* Note: in.type is unsigned */
-			       (m->in.type < SZOF(typ)) ? 
-					typ[m->in.type] : "*bad*",
-			       m->in.offset);
+			       (m->in_type < SZOF(typ)) ? 
+					typ[m->in_type] : "*bad*",
+			       m->in_offset);
 
 	(void) fprintf(stderr, " %s%s", (m->flag & UNSIGNED) ? "u" : "",
 		       /* Note: type is unsigned */
 		       (m->type < SZOF(typ)) ? typ[m->type] : "*bad*");
-	if (m->mask != ~((uint32)0))
-		(void) fprintf(stderr, " & %.8x", m->mask);
+	if (m->mask != ~((uint32)0)) {
+		if(STRING != m->type)
+			(void) fprintf(stderr, " & %.8x", m->mask);
+		else {
+			(void) fputc('/', stderr); 
+			if (m->mask & STRING_IGNORE_LOWERCASE) 
+				(void) fputc(CHAR_IGNORE_LOWERCASE, stderr);
+			if (m->mask & STRING_COMPACT_BLANK) 
+				(void) fputc(CHAR_COMPACT_BLANK, stderr);
+			if (m->mask & STRING_COMPACT_OPTIONAL_BLANK) 
+				(void) fputc(CHAR_COMPACT_OPTIONAL_BLANK,
+				stderr);
+		}
+	}
 
 	(void) fprintf(stderr, ",%c", m->reln);
 
 	if (m->reln != 'x') {
-	    switch (m->type) {
-	    case BYTE:
-	    case SHORT:
-	    case LONG:
-	    case LESHORT:
-	    case LELONG:
-	    case BESHORT:
-	    case BELONG:
-		    (void) fprintf(stderr, "%d", m->value.l);
-		    break;
-	    case STRING:
-		    showstr(stderr, m->value.s, -1);
-		    break;
-	    case DATE:
-	    case LEDATE:
-	    case BEDATE:
-		    {
-			    time_t t = m->value.l;
-			    char *rt, *pp = ctime(&t);
+		switch (m->type) {
+		case BYTE:
+		case SHORT:
+		case LONG:
+		case LESHORT:
+		case LELONG:
+		case BESHORT:
+		case BELONG:
+			(void) fprintf(stderr, "%d", m->value.l);
+			break;
+		case STRING:
+			showstr(stderr, m->value.s, -1);
+			break;
+		case DATE:
+		case LEDATE:
+		case BEDATE:
+			{
+				time_t t = m->value.l;
+				char *rt, *pp = ctime(&t);
 
-			    if ((rt = strchr(pp, '\n')) != NULL)
-				    *rt = '\0';
-			    (void) fprintf(stderr, "%s,", pp);
-			    if (rt)
-				    *rt = '\n';
-		    }
-		    break;
-	    default:
-		    (void) fputs("*bad*", stderr);
-		    break;
-	    }
+				if ((rt = strchr(pp, '\n')) != NULL)
+					*rt = '\0';
+				(void) fprintf(stderr, "%s,", pp);
+				if (rt)
+					*rt = '\n';
+			}
+			break;
+		default:
+			(void) fputs("*bad*", stderr);
+			break;
+		}
 	}
 	(void) fprintf(stderr, ",\"%s\"]\n", m->desc);
 }
@@ -115,8 +127,8 @@ struct magic *m;
  */
 void
 ckfputs(str, fil) 	
-    const char *str;
-    FILE *fil;
+	const char *str;
+	FILE *fil;
 {
 	if (fputs(str,fil) == EOF)
 		error("write failed.\n");
